@@ -37,7 +37,8 @@ struct _TerminalMdiContainerInterface {
 
   /* vfuncs */
   void                  (* add_screen)              (TerminalMdiContainer *container,
-                                                     TerminalScreen *screen);
+                                                     TerminalScreen *screen,
+                                                     int position);
   void                  (* remove_screen)           (TerminalMdiContainer *container,
                                                      TerminalScreen *screen);
   TerminalScreen *      (* get_active_screen)       (TerminalMdiContainer *container);
@@ -47,6 +48,8 @@ struct _TerminalMdiContainerInterface {
   GList *               (* list_screen_containers)  (TerminalMdiContainer *container);
   int                   (* get_n_screens)           (TerminalMdiContainer *container);
   int                   (* get_active_screen_num)   (TerminalMdiContainer *container);
+  void                  (* set_active_screen_num)   (TerminalMdiContainer *container,
+                                                     int position);
   void                  (* reorder_screen)          (TerminalMdiContainer *container,
                                                      TerminalScreen *screen,
                                                      int new_position);
@@ -67,7 +70,8 @@ struct _TerminalMdiContainerInterface {
 GType terminal_mdi_container_get_type (void);
 
 void terminal_mdi_container_add_screen (TerminalMdiContainer *container,
-                                        TerminalScreen *screen);
+                                        TerminalScreen *screen,
+                                        int position);
 
 void terminal_mdi_container_remove_screen (TerminalMdiContainer *container,
                                            TerminalScreen *screen);
@@ -77,6 +81,9 @@ TerminalScreen *terminal_mdi_container_get_active_screen (TerminalMdiContainer *
 void terminal_mdi_container_set_active_screen (TerminalMdiContainer *container,
                                                TerminalScreen *screen);
 
+void terminal_mdi_container_set_active_screen_num (TerminalMdiContainer *container,
+                                                   int position);
+
 GList *terminal_mdi_container_list_screens (TerminalMdiContainer *container);
 
 GList *terminal_mdi_container_list_screen_containers (TerminalMdiContainer *container);
@@ -89,6 +96,9 @@ void terminal_mdi_container_reorder_screen (TerminalMdiContainer *container,
                                             TerminalScreen *screen,
                                             int new_position);
 
+void terminal_mdi_container_change_screen (TerminalMdiContainer *container,
+                                           int change);
+
 G_END_DECLS
 
 #endif /* TERMINAL_MDI_CONTAINER_H */
